@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2014 Samy Al Bahra.
+ * Copyright 2009-2015 Samy Al Bahra.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,10 +24,11 @@
  * SUCH DAMAGE.
  */
 
-#ifndef _CK_BACKOFF_H
-#define _CK_BACKOFF_H
+#ifndef CK_BACKOFF_H
+#define CK_BACKOFF_H
 
 #include <ck_cc.h>
+#include <ck_pr.h>
 
 #ifndef CK_BACKOFF_CEILING
 #define CK_BACKOFF_CEILING ((1 << 20) - 1)
@@ -35,23 +36,22 @@
 
 #define CK_BACKOFF_INITIALIZER (1 << 9)
 
-typedef volatile unsigned int ck_backoff_t;
+typedef unsigned int ck_backoff_t;
 
 /*
  * This is a exponential back-off implementation.
  */
 CK_CC_INLINE static void
-ck_backoff_eb(volatile unsigned int *c)
+ck_backoff_eb(unsigned int *c)
 {
-	volatile unsigned int i;
-	unsigned int ceiling;
+	unsigned int i, ceiling;
 
 	ceiling = *c;
-
-	for (i = 0; i < ceiling; i++);
+	for (i = 0; i < ceiling; i++)
+		ck_pr_barrier();
 
 	*c = ceiling <<= ceiling < CK_BACKOFF_CEILING;
 	return;
 }
 
-#endif /* _CK_BACKOFF_H */
+#endif /* CK_BACKOFF_H */
