@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2014 Samy Al Bahra.
+ * Copyright 2012-2015 Samy Al Bahra.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,16 +24,16 @@
  * SUCH DAMAGE.
  */
 
-#ifndef _CK_HS_H
-#define _CK_HS_H
+#ifndef CK_HS_H
+#define CK_HS_H
 
 #include <ck_cc.h>
 #include <ck_malloc.h>
 #include <ck_md.h>
 #include <ck_pr.h>
 #include <ck_stdint.h>
-#include <stdbool.h>
-#include <stddef.h>
+#include <ck_stdbool.h>
+#include <ck_stddef.h>
 
 /*
  * Indicates a single-writer many-reader workload. Mutually
@@ -131,4 +131,4 @@ bool ck_hs_reset(ck_hs_t *);
 bool ck_hs_reset_size(ck_hs_t *, unsigned long);
 void ck_hs_stat(ck_hs_t *, struct ck_hs_stat *);
 
-#endif /* _CK_HS_H */
+#endif /* CK_HS_H */
