@@ -124,7 +124,7 @@ hs_free(void *p, size_t b, bool r)
 
 	if (r == true) {
 		/* Destruction requires safe memory reclamation. */
-		ck_epoch_call(&epoch_hs, &epoch_wr, &(--e)->epoch_entry, hs_destroy);
+		ck_epoch_call(&epoch_wr, &(--e)->epoch_entry, hs_destroy);
 	} else {
 		free(--e);
 	}
@@ -237,7 +237,7 @@ reader(void *unused)
 	ck_epoch_register(&epoch_hs, &epoch_record);
 	for (;;) {
 		j++;
-		ck_epoch_begin(&epoch_hs, &epoch_record);
+		ck_epoch_begin(&epoch_record, NULL);
 		s = rdtsc();
 		for (i = 0; i < keys_length; i++) {
 			char *r;
@@ -257,7 +257,7 @@ reader(void *unused)
 			ck_error("ERROR: Found invalid value: [%s] but expected [%s]\n", (char *)r, keys[i]);
 		}
 		a += rdtsc() - s;
-		ck_epoch_end(&epoch_hs, &epoch_record);
+		ck_epoch_end(&epoch_record, NULL);
 
 		n_state = ck_pr_load_int(&state);
 		if (n_state != state_previous) {
@@ -452,7 +452,7 @@ main(int argc, char *argv[])
 	fprintf(stderr, "done (%" PRIu64 " ticks)\n", a / (r * keys_length));
 
 	ck_epoch_record_t epoch_temporary = epoch_wr;
-	ck_epoch_synchronize(&epoch_hs, &epoch_wr);
+	ck_epoch_synchronize(&epoch_wr);
 
 	fprintf(stderr, " '- Summary: %u pending, %u peak, %lu reclamations -> "
 	    "%u pending, %u peak, %lu reclamations\n\n",
@@ -503,7 +503,7 @@ main(int argc, char *argv[])
 	while (ck_pr_load_int(&barrier[HS_STATE_STRICT_REPLACEMENT]) != n_threads)
 		ck_pr_stall();
 	set_reset();
-	ck_epoch_synchronize(&epoch_hs, &epoch_wr);
+	ck_epoch_synchronize(&epoch_wr);
 	fprintf(stderr, "done (writer = %" PRIu64 " ticks, reader = %" PRIu64 " ticks)\n",
 	    a / (repeated * keys_length), acc(HS_STATE_STRICT_REPLACEMENT) / n_threads);
 
@@ -538,7 +538,7 @@ main(int argc, char *argv[])
 		ck_pr_stall();
 
 	set_reset();
-	ck_epoch_synchronize(&epoch_hs, &epoch_wr);
+	ck_epoch_synchronize(&epoch_wr);
 	fprintf(stderr, "done (writer = %" PRIu64 " ticks, reader = %" PRIu64 " ticks)\n",
 	    a / (repeated * keys_length), acc(HS_STATE_DELETION) / n_threads);
 
@@ -585,13 +585,13 @@ main(int argc, char *argv[])
 	while (ck_pr_load_int(&barrier[HS_STATE_REPLACEMENT]) != n_threads)
 		ck_pr_stall();
 	set_reset();
-	ck_epoch_synchronize(&epoch_hs, &epoch_wr);
+	ck_epoch_synchronize(&epoch_wr);
 	fprintf(stderr, "done (writer = %" PRIu64 " ticks, reader = %" PRIu64 " ticks)\n",
 	    a / (repeated * keys_length), acc(HS_STATE_REPLACEMENT) / n_threads);
 
 	ck_pr_inc_int(&barrier[HS_STATE_REPLACEMENT]);
 	epoch_temporary = epoch_wr;
-	ck_epoch_synchronize(&epoch_hs, &epoch_wr);
+	ck_epoch_synchronize(&epoch_wr);
 
 	fprintf(stderr, " '- Summary: %u pending, %u peak, %lu reclamations -> "
 	    "%u pending, %u peak, %lu reclamations\n\n",
